<?php

$has_error = false;
$error_message = null;

if (isset($_POST["submitted"])) {
	$user = new NewUser($_POST);

	try {
		$user->makeVerifications();
		$user->register();
	} catch (AssertionError $e) {
		$has_error = true;
		$error_message = $e->getMessage();
	}
}

class NewUser
{
	public $email;
	public $first_name;
	public $surname;
	public $role;
	public $school;
	public $class;

	public $description;
	public $confirm_email_token;
	private $password;
	private $confirm_password;

	public function __construct($data)
	{
		foreach ($data as $key => $value)
			$this->$key = htmlspecialchars($value);
	}

	public function makeVerifications()
	{
		global $CONFIG;

		ensure(date("Y-m-d H:i:s") < $CONFIG->getInscriptionDate(), "Les inscriptions sont terminées.");
		ensure(filter_var($this->email, FILTER_VALIDATE_EMAIL), "L'adresse e-mail entrée est invalide.");
		$this->email = strtolower($this->email);
		ensure(!userExists($this->email), "Un compte existe déjà avec cette adresse e-mail.");
		ensure(strlen($this->password) >= 8, "Le mot de passe doit comporter au moins 8 caractères.");
		ensure($this->password == $this->confirm_password, "Les deux mots de passe sont différents.");
		ensure($this->surname != "", "Le nom de famille est obligatoire.");
		ensure($this->first_name != "", "Le prénom est obligatoire.");
		$this->role = Role::fromName(strtoupper($this->role));

		if ($this->role == Role::PARTICIPANT)
			$this->class = SchoolClass::fromName(strtoupper($this->class));

		$this->confirm_email_token = genRandomPhrase(64);
	}

	public function register()
	{
		global $DB, $YEAR;

		$req = $DB->prepare("INSERT INTO `users`(`email`, `pwd_hash`, `confirm_email`, `surname`, `first_name`, `school`, `class`, `role`, `description`, `year`)
                VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?);");
		$req->execute([$this->email, password_hash($this->password, PASSWORD_BCRYPT), $this->confirm_email_token, $this->surname, $this->first_name,
			$this->school, SchoolClass::getName($this->class), Role::getName($this->role), $this->description, $YEAR]);

		Mailer::sendRegisterMail($this);
	}
}

require_once "server_files/views/inscription.php";
