<?php

if (!isset($_SESSION["user_id"]))
	require_once "server_files/403.php";

/** @var User $user */
$user = $_SESSION["user"];

$has_error = false;
$error_message = null;

if (isset($_POST["submitted"])) {
    $my_account = new MyAccount($_POST);
    try {
    	$my_account->makeVerifications();
    	$my_account->updateAccount();
	}
	catch (AssertionError $e) {
    	$has_error = true;
    	$error_message = $e->getMessage();
	}
}

if (isset($_POST["submitted_password"])) {
	$new_password = new NewPassword($_POST);
	try {
		$new_password->makeVerifications();
		$new_password->updatePassword();
	}
	catch (AssertionError $e) {
		$has_error = true;
		$error_message = $e->getMessage();
	}
}

class MyAccount
{
	public $email;
	public $surname;
	public $first_name;
	public $school;
	public $class;
	public $description;
	private $user;

	public function __construct($data)
	{
		foreach ($data as $key => $value)
			$this->$key = htmlspecialchars($value);

		$this->user = $_SESSION["user"];

		$keys = ["email", "surname", "first_name", "school", "class", "description"];

		if ($this->user->getRole() == Role::PARTICIPANT)
			$this->class = SchoolClass::fromName($this->class);

		foreach ($keys as $key)
			$this->$key = $this->$key != null && $this->$key != "" ? $this->$key : $this->user->$key;
	}

	public function makeVerifications()
	{
		ensure(filter_var($this->email, FILTER_VALIDATE_EMAIL), "L'adresse e-mail entrée est invalide.");
		$this->email = strtolower($this->email);
		ensure($this->email == $this->user->getEmail() || !userExists($this->email), "Un compte existe déjà avec cette adresse e-mail.");
	}

	public function updateAccount()
	{
		$this->user->setSurname($this->surname);
		$this->user->setFirstName($this->first_name);
		$this->user->setSchool($this->school);
		$this->user->setClass($this->class);
		$this->user->setDescription($this->description);

		if ($this->email != $this->user->getEmail()) {
			$this->user->setEmail($this->email);
			$this->user->setConfirmEmailToken(genRandomPhrase(64));

			Mailer::sendChangeEmailAddressMail($this->user);
		}
	}
}

class NewPassword
{
	private $user;
	private $old_password;
	private $new_password;
	private $confirm_password;

	public function __construct($data)
	{
		foreach ($data as $key => $value)
			$this->$key = htmlspecialchars($value);

		$this->user = $_SESSION["user"];
	}

	public function makeVerifications()
	{
		ensure($this->user->checkPassword($this->old_password), "L'ancien mot de passe est incorrect.");
		ensure(strlen($this->new_password) >= 8, "Le mot de passe doit comporter au moins 8 caractères.");
		ensure($this->new_password == $this->confirm_password, "Les deux mots de passe sont différents.");
	}

	public function updatePassword()
	{
		$this->user->setPassword($this->new_password);

		Mailer::sendChangePasswordMail($this->user);
	}
}

require_once "server_files/views/mon_compte.php";
